#!/usr/bin/python

import numpy
import scipy.stats

def read_waterfall(filename) :
    blocks = []
    data = []
    for line in file(filename, 'r') :
        if line == "\n" : # end of block
            if len(data) > 0 :
                blocks.append(numpy.array(data))
                data = []
        else :
            parsed = line.split('\n')[0].split()
            x = float(parsed[0])
            y = float(parsed[1])
            data.append((x,y))
    if len(data) > 0 :
        blocks.append(numpy.array(data))
    return blocks

def fit_block(block, F=None) :
    logx = numpy.log2(block[:,0])
    y = block[:,1]
    gradient, intercept, r_value, p_value, std_err = scipy.stats.linregress(logx, y)
    print "fit:\t%g * ln(rate) + %g\n" % (gradient/numpy.log(2), intercept) # numpy.log is base e
    if F != None :
        # determine rate to generate this force
        lx = float(F-intercept)/gradient
        print " F = %g for rate = %g" % (F, 2**lx)
    def fit_eval(logx) :
        x = 2**logx
        y = gradient * logx + intercept
        return (x,y)
    points = [fit_eval(logx.min()), fit_eval(logx.max())]
    return numpy.array(points)

def print_mp_block(fid, block) :
    for i in range(block.shape[0]) :
        fid.write("%g\t%g\n" % (block[i][0], block[i][1]))
    fid.write("\n")

def print_mp_data(filename, blocks, fits) :
    fid = file(filename, 'w')
    for block, fit in zip(blocks, fits) :
        print_mp_block(fid, block)
        print_mp_block(fid, fit)
    fid.close()

if __name__ == "__main__" :
    import sys

    fname = sys.argv[1]
    blocks = read_waterfall(fname)
    fits = []
    for block in blocks :
        fits.append( fit_block(block, F=170) )
    print_mp_data('unfolding_speed.d', blocks, fits)
